"""File system utility."""
import contextlib
import os
import tempfile
from pathlib import Path
from typing import Generator
from typing import Union

PathLike = Union[str, bytes, os.PathLike]  #: Path-like object type


def _add_ext(f_name: PathLike, ext: str) -> str:
    if isinstance(f_name, bytes):
        f_name_str = f_name.decode()
    else:
        f_name_str = str(f_name)

    if not f_name_str.endswith(ext):
        f_name_str = f"{f_name_str}{ext}"
    return f_name_str


def to_path(path: PathLike) -> Path:
    """Converts a PathLike path to a Path.

    Args:
        path: Path-like object.
    """
    if isinstance(path, bytes):
        path = path.decode()
    return Path(path)


def create_dir(d_name: PathLike) -> None:
    """Creates a directory if it does not yet exist.

    Also creates its parents if they do not yet exist.

    Args:
        d_name: Path to the directory to create.
    """
    d_name_path = to_path(d_name)
    if not d_name_path.is_dir():
        d_name_path.mkdir(exist_ok=True, parents=True)


@contextlib.contextmanager
def temp_dir_path() -> Generator[Path, None, None]:
    """Yields a temporary directory as Path."""
    with tempfile.TemporaryDirectory() as temp_dir:
        temp_path = Path(temp_dir)
        yield temp_path
